Module component

Module component 

The Component trait (framework-internal) and built-in containers (VStack, HStack, Column).

Structs

Column
Width-constrained wrapper for use inside HStack.
HStack
Horizontal stack container.
Tracked
Wrapper that automatically marks component state dirty on mutation.
VStack
Vertical stack container.

Enums

EventResult
Result of handling an input event in a component’s event handler.

Traits

Component
A component that can render itself into a terminal region.